local MASQUE = LibStub("Masque", true);
local buttongroupItems = nil;
    
CleanUIContainerLock = CleanUIContainerLock or {}

local ItemUpgradeInfo = LibStub("LibItemUpgradeInfo-1.0")

-- temporary data
local item_profit = 0;

function CleanUI_InitBags()

    CleanUI_BagsHookEvents();
    CleanUI_RegisterBagsEvents();

    if (MASQUE) then
        buttongroupItems = MASQUE:Group("CleanUI", "ItemBar");
        
        CleanUI_SkinActionBars();
        CleanUI_SkinBagButtons();
    end

    -- Display the amount of copper that is added to the tooltip
    GameTooltip:HookScript("OnTooltipAddMoney",
        function(self, amount)
            item_profit = amount;
        end);
    
    CleanUI_CreateItemButtons();
    
    BackpackTokenFrameToken1Icon:SetTexCoord(0.1, 0.9, 0.1, 0.9);
    BackpackTokenFrameToken2Icon:SetTexCoord(0.1, 0.9, 0.1, 0.9);
    BackpackTokenFrameToken3Icon:SetTexCoord(0.1, 0.9, 0.1, 0.9);
end

function CleanUI_RegisterBagsEvents()
    local frame = CreateFrame("Frame");

    frame:RegisterEvent("ADDON_LOADED");
    frame:RegisterEvent("MERCHANT_SHOW");
    
    frame:HookScript("OnEvent", CleanUI_BagsOnEvent);
end

function CleanUI_BagsOnEvent(self, event, ...)

    if (event == "ADDON_LOADED") then
        local addonName = ...;
        if ( addonName == "Blizzard_GuildBankUI" ) then
        end
    end

    if (event == "MERCHANT_SHOW") then
        CleanUI_MerchantShow();
    end
end

function CleanUI_BagsHookEvents()
    hooksecurefunc("ContainerFrame_Update", CleanUI_ContainerFrame_Update);
    hooksecurefunc("ContainerFrameItemButton_OnModifiedClick", CleanUI_ContainerFrameItemButton_OnModifiedClick);

    hooksecurefunc("ContainerFrame_GenerateFrame", CleanUI_ContainerFrame_GenerateFrame);
    
    -- Tooltip für MoneyFrame   
    ContainerFrame1MoneyFrameGoldButton:HookScript("OnEnter", CleanUI_ShowMoneyFrameTooltip);
    ContainerFrame1MoneyFrameGoldButton:HookScript("OnLeave", CleanUI_HideMoneyFrameTooltip);      
    ContainerFrame1MoneyFrameSilverButton:HookScript("OnEnter", CleanUI_ShowMoneyFrameTooltip);
    ContainerFrame1MoneyFrameSilverButton:HookScript("OnLeave", CleanUI_HideMoneyFrameTooltip);  
    ContainerFrame1MoneyFrameCopperButton:HookScript("OnEnter", CleanUI_ShowMoneyFrameTooltip);
    ContainerFrame1MoneyFrameCopperButton:HookScript("OnLeave", CleanUI_HideMoneyFrameTooltip);
end

function CleanUI_ContainerFrameItemButton_OnModifiedClick(self, button)
    if (not IsAltKeyDown()) then
        return;
    end

    if (button == "LeftButton") then
        -- item lock
        local itemLink = GetContainerItemLink(self:GetParent():GetID(), self:GetID());

        if (itemLink) then
            -- lock state can't be removed by modified click
            if (not CleanUI_IsLockedItem(itemLink)) then
                CleanUI_AddLockedItem(itemLink);
            end
        end

        CleanUI_UpdateAllContainers();
    elseif (button == "RightButton") then
        -- sell junk

        CleanUI_UpdateAllContainers();
    end
end

function CleanUI_UpdateAllContainers()
    for i=1, NUM_CONTAINER_FRAMES, 1 do
        local frame = _G["ContainerFrame"..i];
        if (frame and frame:IsShown()) then
            ContainerFrame_Update(frame);
        end
    end
end

function CleanUI_IsLockedItem(link)
    local itemName = GetItemInfo(link);
    local foundItemName;

    for index, foundLink in ipairs(CleanUIContainerLock) do
        foundItemName = GetItemInfo(foundLink);

        if (foundItemName == itemName) then
            return true;
        end
    end

    return false;
end

function CleanUI_AddLockedItem(link)
    tinsert(CleanUIContainerLock, link);
end

function CleanUI_RemoveLockedItem(link)
    local itemName = GetItemInfo(link);
    local foundItemName;

    for index, foundLink in ipairs(CleanUIContainerLock) do
        foundItemName = GetItemInfo(foundLink);

        if (foundItemName == itemName) then
            tremove(CleanUIContainerLock, index);

            CleanUI_UpdateAllContainers();
            return true;
        end
    end
end

function CleanUI_MerchantShow()
    -- auto repair
    if (not IsControlKeyDown() and CanMerchantRepair()) then
        repairAllCost, canRepair = GetRepairAllCost();

        if (IsAltKeyDown()) then
            RepairAllItems(1);      -- guild
        else
            RepairAllItems();
        end
        
        PlaySound("ITEM_REPAIR");

        if (repairAllCost > 0) then
            DEFAULT_CHAT_FRAME:AddMessage("Rüstung wurde repariert. ("..GetCoinTextureString(repairAllCost)..")", 1.0, 1.0, 0, 1, 10);
        end
    end

    local sum_item_profit = 0

    local container, containerid, numberOfSlots;
    local itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemTexture;
    for id = 0, 4, 1 do
        containerid = "ContainerFrame"..(id + 1);
        container = getglobal(containerid);

        if (container) then
            numberOfSlots = GetContainerNumSlots(id);

            for slot=1, numberOfSlots, 1 do
                itemLink = GetContainerItemLink(id, numberOfSlots - slot + 1);

                if (itemLink) then
                    itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemTexture = GetItemInfo(itemLink);

                    if (itemRarity == 0) then
                        GameTooltip:SetBagItem(id, numberOfSlots - slot + 1);

                        if (item_profit and item_profit > 0) then
                            sum_item_profit = sum_item_profit + item_profit;
                            DEFAULT_CHAT_FRAME:AddMessage(itemLink.. " wurde verkauft. ("..GetCoinTextureString(item_profit)..")", 1.0, 1.0, 0, 1, 10);
                            UseContainerItem(id, numberOfSlots - slot + 1);
                        end
                    end
                end
            end
        end
    end

    if (sum_item_profit > 0) then
        DEFAULT_CHAT_FRAME:AddMessage("Gesamt: "..GetCoinTextureString(sum_item_profit), 1.0, 1.0, 0, 1, 10);
    end
end

function CleanUI_ContainerFrame_Update(frame)
    local id = frame:GetID();
    local name = frame:GetName();

    local itemButton, itemLink;

    for i=1, frame.size, 1 do
        itemButton = _G[name.."Item"..i];
        itemLink = GetContainerItemLink(id, itemButton:GetID());

        CleanUI_UpdateItemLock(itemButton, itemLink);
        CleanUI_ContainerFrame_UpdateItemLevel(itemButton, itemLink);
    end
end

function CleanUI_UpdateItemLock(itemButton, itemLink)
    if (not itemLink) then
        CleanUI_UnLockItem(itemButton)
        return;
    end

    if (CleanUI_IsLockedItem(itemLink)) then
        CleanUI_LockItem(itemButton);
    else
        CleanUI_UnLockItem(itemButton);
    end

    if (IsEquippableItem(itemLink)) then
        local itemIDs, foundLink, found;
        for index = 1, GetNumEquipmentSets() do
            local eqname, icon = C_EquipmentSet.GetEquipmentSetInfo(index);
            found = false;

            itemIDs = GetEquipmentSetItemIDs(eqname);

            for _, itemID in ipairs(itemIDs) do
                if (itemID > 0) then
                    _, foundLink = GetItemInfo(itemID);

                    local itemName = GetItemInfo(itemLink);
                    local foundName = GetItemInfo(foundLink);
                    
                    if (itemName == foundName) then
                        found = true;
                        break;
                    end
                end
            end
        end

        CleanUI_UpdateEquipmentSetMarker(itemButton, found);
    end
end

function CleanUI_UpdateEquipmentSetMarker(itemButton, found)
    if (not itemButton.eqIndicator) then
        itemButton.eqIndicator = itemButton:CreateFontString(itemButton:GetName().."EQIndicator", "ARTWORK");
        itemButton.eqIndicator:SetFontObject("NumberFontNormalSmallGray");
        itemButton.eqIndicator:SetSize(16, 16);
        itemButton.eqIndicator:ClearAllPoints();
        itemButton.eqIndicator:SetPoint("TOPLEFT", -1, 3);
        itemButton.eqIndicator:SetJustifyH("CENTER");
        itemButton.eqIndicator:SetText(NORMAL_FONT_COLOR_CODE.."●"..FONT_COLOR_CODE_CLOSE);
    end
    
    if (found) then
        itemButton.eqIndicator:Show();
    else
        itemButton.eqIndicator:Hide();
    end
end

function CleanUI_ContainerFrame_UpdateItemLevel(self, itemLink)
    -- create label
    local itemLevelLabel = _G[self:GetName().."ItemLevel"];
    if (not itemLevelLabel) then
        itemLevelLabel = self:CreateFontString(self:GetName().."ItemLevel", "ARTWORK");
        itemLevelLabel:SetFontObject("GameFontNormalSmall");
        itemLevelLabel:SetSize(self:GetWidth(), 10);
        itemLevelLabel:ClearAllPoints();
        itemLevelLabel:SetPoint("TOPRIGHT", -2, -2);
        itemLevelLabel:SetJustifyH("RIGHT");
        itemLevelLabel:SetText("");
    end
    
    -- no item link, no label
    if (itemLink == nil) then
        itemLevelLabel:Hide();
        return;
    end
    
    local _, _, quality, _, _, _, _, _, equipSlot = GetItemInfo(itemLink);
    
    -- no item link, no label
    if (string.find(equipSlot, "INVTYPE") ~= 1) then
        itemLevelLabel:Hide();
        return;
    end
    
    -- show
    itemLevelLabel:Show();
    
    -- calculate level
    local itemLevel = ItemUpgradeInfo:GetUpgradedItemLevel(itemLink);
    
    -- colorize
    local color = ITEM_QUALITY_COLORS[quality];
    local hexcolor = cui_rgb2hex(color.r, color.g, color.b);
    
    -- set text
    itemLevelLabel:SetText("|cff"..hexcolor..itemLevel.."|r");
end

function CleanUI_LockItem(itemButton)
    if (not itemButton:GetParent() or string.find(itemButton:GetParent():GetName(), "ContainerFrame") == nil) then
        return;
    end
    
    itemButton:RegisterForClicks();
    itemButton:RegisterForDrag();

    if (not itemButton.unlockButton) then
        itemButton.unlockButton = CreateFrame("Button", nil, itemButton, "UIPanelButtonTemplate");
        CleanUI_RemoveUiButtonTexture(itemButton.unlockButton);
        itemButton.unlockButton:SetSize(16, 16);
        itemButton.unlockButton:SetNormalTexture("Interface\\PETBATTLES\\PetBattle-LockIcon");
        itemButton.unlockButton:GetNormalTexture():SetTexCoord(0, 1, 0, 1);
        itemButton.unlockButton:SetPoint("TOPLEFT", itemButton, "TOPLEFT", -1, 1);

        itemButton.unlockButton:SetScript("OnClick", CleanUI_UnLockClicked);
    end

    itemButton.unlockButton:Show();
end

function CleanUI_UnLockClicked(unlockButton)
    if (IsAltKeyDown()) then
        local itemLink = GetContainerItemLink(unlockButton:GetParent():GetParent():GetID(), unlockButton:GetParent():GetID());
        CleanUI_RemoveLockedItem(itemLink);
    end
end

function CleanUI_UnLockItem(itemButton)
    if (not itemButton:GetParent() or string.find(itemButton:GetParent():GetName(), "ContainerFrame") == nil) then
        return;
    end
    
    itemButton:RegisterForClicks("LeftButtonUp", "RightButtonUp");
    itemButton:RegisterForDrag("LeftButton");

    if (itemButton.unlockButton) then
        itemButton.unlockButton:Hide();
    end
end

function CleanUI_ContainerFrame_GenerateFrame(frame, size, id)
    frame:SetFrameStrata("HIGH");
end

function CleanUI_ShowMoneyFrameTooltip(self)
    GameTooltip_SetDefaultAnchor(GameTooltip, self);
    GameTooltip:SetText(MONEY..":", 1.0, 1.0, 1.0);
    GameTooltip:AddLine(" ");
    
    local data = CleanUICurrency["money"] or {};
    
    local summe = 0;
    local name, moneyString;
    
    for guid, amount in pairs(data) do
        summe = summe + amount;
        
        name = "|c333399ff"..CleanUIAccountData[guid].name.."|r";
        
        moneyString = "|cffffffff"..GetCoinTextureString(amount).."|r";
        
        GameTooltip:AddDoubleLine(name, moneyString);
    end
    
    GameTooltip:AddLine(" ");
    
    GameTooltip:AddDoubleLine("|c333399ffAccount|r", "|cffffffff"..GetCoinTextureString(summe).."|r");

    GameTooltip:Show();
end

function CleanUI_HideMoneyFrameTooltip()
    GameTooltip:FadeOut();
end

function CleanUI_CreateItemButtons()
    if (not CleanUISavedVariables.ItemButtons) then
        return;
    end
    
    --CleanUI_CreateItemButton("CleanUIHearthstoneButton", 6948, 0);
    CleanUI_CreateItemButton("CleanUIFoliantDesStadtportalsButton", 142542, 0);
    CleanUI_CreateItemButton("CleanUIDalaranHearthstoneButton", 140192, 40);
    
    CleanUI_CreateItemButton("CleanUIHimmelshorndracheButton", 131811, 95);
    CleanUI_CreateItemButton("CleanUIPfeifeSmaragdwindeButton", 139773, 135);
    
    CleanUI_CreateItemButton("CleanUIPfeifeFlugmeisterButton", 141605, 190);
    
    CleanUI_CreateItemButton("CleanUIKuerschnermesserButton", 130102, 240);
end

function CleanUI_BagUpdateCooldown(button)
    local cooldown = button.cooldown;
    local start, duration, enable = GetItemCooldown(button:GetAttribute("itemId"));
    CooldownFrame_Set(cooldown, start, duration, enable);
end

function CleanUI_CreateItemButton(name, id, offset)
    local button = _G[name];
    if (not button) then
        button = CreateFrame("Button", name, UIParent, "SecureActionButtonTemplate, ActionButtonTemplate");
        button:RegisterEvent("ACTIONBAR_UPDATE_COOLDOWN");
        button:RegisterEvent("BAG_UPDATE");
    
        button:RegisterEvent("PLAYER_REGEN_DISABLED");
        button:RegisterEvent("PLAYER_REGEN_ENABLED");
    end
    
    local itemTexture = GetItemIcon(id); 
    local itemName, itemLink = GetItemInfo(id);
    button.icon:SetTexture(itemTexture);
    
    button:SetPoint("TOPRIGHT", -1, -250 - offset);
    button:SetFrameStrata("LOW");
    
    buttongroupItems:AddButton(button);
    
    button:SetAttribute("type", "item");
    button:SetAttribute("item", itemName);
    button:SetAttribute("itemId", id);
    button:SetAttribute("itemLink", itemLink);

    button.cooldown = _G[button:GetName().."Cooldown"];
    button.cooldown:ClearAllPoints();
    button.cooldown:SetPoint("TOPLEFT", button, "TOPLEFT", 0, -0);
    button.cooldown:SetPoint("BOTTOMRIGHT", button, "BOTTOMRIGHT", -0, 0);
    
    button.cooldown:SetEdgeTexture("Interface\\Cooldown\\edge");
    button.cooldown:SetSwipeColor(0, 0, 0);
    button.cooldown:SetHideCountdownNumbers(false);
    button.cooldown.currentCooldownType = COOLDOWN_TYPE_NORMAL;
    
    button:HookScript("OnEnter", CleanUI_ShowBagItemTooltip);
    button:HookScript("OnLeave", CleanUI_HideBagItemTooltip);
    
    button:HookScript("OnEvent", CleanUI_ItemButton_OnEvent);  
    
    CleanUI_CheckBagItemUsable(button);
end

function CleanUI_ItemButton_OnEvent(self, event, ...)
    if ( event == "ACTIONBAR_UPDATE_COOLDOWN" ) then
        CleanUI_BagUpdateCooldown(self);
    end
    if ( event == "BAG_UPDATE" or event == "PLAYER_REGEN_DISABLED" or event == "PLAYER_REGEN_ENABLED" ) then
        CleanUI_CheckBagItemUsable(self);
    end
end

function CleanUI_ShowBagItemTooltip(self)
    GameTooltip_SetDefaultAnchor(GameTooltip, self);
    GameTooltip:SetItemByID(self:GetAttribute("itemId"));

    GameTooltip:Show();
end

local function findItemByName(search)
    if (not search) then
        return false;
    end
    
    for bag = 0,4 do
        for slot = 1,GetContainerNumSlots(bag) do
            local item = GetContainerItemLink(bag,slot)
            if item and item:find(search) then
                return true;
            end
        end
    end
    
    return false;
end

function CleanUI_CheckBagItemUsable(self)
    
    if (not InCombatLockdown()) then
    
        local owned = findItemByName(self:GetAttribute("item"))
        local isUsable = IsUsableItem(self:GetAttribute("itemId"))
        
        -- FIXME toy fix
        if (self:GetAttribute("itemId") == 131811
            or self:GetAttribute("itemId") == 139773
            or self:GetAttribute("itemId") == 142542)  then
            owned = true;
        end
    
        if (owned) then
            self:Show();
            if (isUsable) then
                self:SetAlpha(1.0);
            else
                self:SetAlpha(0.8);
            end
        else
            self:Hide();
        end 
    end

    GameTooltip:Show();
end

function CleanUI_HideBagItemTooltip()
    GameTooltip:FadeOut();
end